package com.leetsketcher.headerizer;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import java.util.ArrayList;

public final class Headerizer {
    public static void printUsage() {
        System.out.println("Usage: java Headerizer [-u] INFILE [OUTFILE|-s]");
        System.out.println("\n\t-u\tUnheaderize the patch");
        System.out.println("\t-s\tOverwrite INFILE");
        System.exit(0);
    }
	
    public static int readByte(FileInputStream in) throws IOException {
        int read = in.read();
        if (read < 0) {
            System.err.println("EOF error: reached the end of the file.");
            System.exit(1);
        }
        return read;
    }

    public static void main(String[] args) throws IOException {
        System.out.println("Headerizer v1.5 by Leet Sketcher (13375K37CH3R)");
        if (args.length == 0) {
            printUsage();
        }
        int index = 0, offset = 2;
        if (args[0].equals("-u")) {
            offset = -2;
            index++;
        }
        if (args.length == index) {
            printUsage();
        }
        if (!args[index].toLowerCase().endsWith(".ips")) {
            throw new IllegalArgumentException("The source file must be a .ips file.");
        }
        ArrayList<Integer> writeBytes = new ArrayList<>();
        try (FileInputStream in = new FileInputStream(args[index])) {
            for (int i = 0; i < 5; i++) writeBytes.add(in.read());
            while (true) {
                    int location = readByte(in) << 8;
                    location |= readByte(in);
                    int low = readByte(in);
                    if (location == 0x454f && low == 0x46) break;
                    location += offset;
                    writeBytes.add(location >> 8);
                    writeBytes.add(location & 0xFF);
                    writeBytes.add(low);
                    int length = readByte(in);
                    writeBytes.add(length);
                    length <<= 8;
                    length |= readByte(in);
                    writeBytes.add(length & 0xFF);
                    if (length == 0) length = 3;
                    for (int i = 0; i < length; i++) writeBytes.add(readByte(in));
            }
            writeBytes.add(0x45);
            writeBytes.add(0x4F);
            writeBytes.add(0x46);
            byte[] truncate = new byte[3];
            if (in.read(truncate) == truncate.length) {
                writeBytes.add(truncate[0] + 0);
                writeBytes.add(truncate[1] + offset);
                writeBytes.add(truncate[2] + 0);
            }
        } catch (FileNotFoundException e) {
            System.err.println("File not found!");
            System.exit(2);
        }
        String outputFile;
        if (args.length - index == 1)
            outputFile = args[index].substring(0, args[index].lastIndexOf("."))
                    + (offset < 0 ? "-N" : "-H") + ".ips";
        else if (args[index + 1].equals("-s")) outputFile = args[index];
        else outputFile = args[index + 1];
        if (!outputFile.toLowerCase().endsWith(".ips")) {
            throw new IllegalArgumentException("The output file must be a .ips file.");
        }
        try (FileOutputStream out = new FileOutputStream(outputFile)) {
            for (int outByte: writeBytes) out.write(outByte);
        }
        System.out.print("The patch was successfully ");
        if (offset < 0) System.out.print("un");
        System.out.println("headerized.");
    }
}